/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: Rule.cc,v 1.19 2002/05/10 07:11:53 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifdef __MINGW32__
#include <assert.h>
#endif

#include <fwbuilder/FWObjectDatabase.hh>
#include <fwbuilder/Rule.hh>
#include <fwbuilder/RuleElement.hh>
#include <fwbuilder/Policy.hh>
#include <fwbuilder/FWOptions.hh>
#include <fwbuilder/XMLTools.hh>

#include <fwbuilder/Policy.hh>

using namespace std;
using namespace libfwbuilder;
 
const char *Rule::TYPENAME={"Rule"};

Rule::Rule()
{
    remStr("name");
    setInt("position",0);
    enable();
}

Rule::Rule(const FWObject *root)
{
    remStr("name");
    setInt("position",0);
    enable();
}

FWOptions* Rule::getOptionsObject()  {    return NULL;              }
void       Rule::setPosition(int n)  {    setInt("position",n);     }
int        Rule::getPosition() const {    return getInt("position");}
void       Rule::disable()           {    setBool("disabled",true); }
void       Rule::enable()            {    setBool("disabled",false);}
bool       Rule::isDisabled()        {    return( getBool("disabled") );}
bool       Rule::isEmpty()           {    return false;             }

/***************************************************************************/

const char *PolicyRule::TYPENAME={"PolicyRule"};

PolicyRule::PolicyRule()
{
    setStr("action","Deny");
    setBool("log",true);
}

PolicyRule::PolicyRule(const FWObject *root) : Rule(root)
{
    setStr("action","Deny");
    setBool("log",true);

    FWObject         *re;
    FWObjectDatabase *db=(FWObjectDatabase*)root;
    assert(db);

    re=db->create("Src",true);  assert(re!=NULL); add(re);
    re=db->create("Dst",true);  assert(re!=NULL); add(re);
    re=db->create("Srv",true);  assert(re!=NULL); add(re);
    re=db->create("When",true); assert(re!=NULL); add(re);

    add( db->create(PolicyRuleOptions::TYPENAME,true) );
}


RuleElementSrc*  PolicyRule::getSrc()  const
{
    return RuleElementSrc::cast(getFirstByType(RuleElementSrc::TYPENAME));
}

RuleElementDst*  PolicyRule::getDst()  const
{
    return RuleElementDst::cast(getFirstByType(RuleElementDst::TYPENAME));
}

RuleElementSrv*  PolicyRule::getSrv()  const
{
    return RuleElementSrv::cast(getFirstByType(RuleElementSrv::TYPENAME));
}

RuleElementInterval* PolicyRule::getWhen()  const
{
    return RuleElementInterval::cast(getFirstByType(RuleElementInterval::TYPENAME));
}


bool PolicyRule::isEmpty()
{
    return (getSrc()->isAny() && getDst()->isAny() && getSrv()->isAny());
}

PolicyRule::Action PolicyRule::getAction()  const
{
    string a=getActionAsString();
    if (a=="Accept")   return Accept;
    if (a=="Deny")     return Deny;
    if (a=="Reject")   return Reject;
    if (a=="Scrub")    return Scrub;
    if (a=="Return")   return Return;
    if (a=="Skip")     return Skip;
    if (a=="Continue") return Continue;
    return Unknown;
}

void   PolicyRule::setAction(PolicyRule::Action act)
{
    switch (act) {
    case Accept:   setAction("Accept");   return;
    case Deny:     setAction("Deny");     return;
    case Reject:   setAction("Reject");   return;
    case Scrub:    setAction("Scrub");    return;
    case Return:   setAction("Return");   return;
    case Skip:     setAction("Skip");     return;
    case Continue: setAction("Continue"); return;
    default:       setAction("Unknown");  return;
    }
}

string PolicyRule::getActionAsString() const    { return getStr("action"); }
void   PolicyRule::setAction(const string& act) { setStr("action",act);    }

PolicyRule::Direction PolicyRule::getDirection() const
{
    string d=getDirectionAsString();
    if (d=="Inbound")   return Inbound;
    if (d=="Outbound")  return Outbound;
    if (d=="Both")      return Both;
    return Undefined;
}

void   PolicyRule::setDirection(PolicyRule::Direction dir)
{
    switch (dir) {
    case Inbound:   setDirection("Inbound");  return;
    case Outbound:  setDirection("Outbound"); return;
    case Both:      setDirection("Both");     return;
    case Undefined: setDirection("");         return;
    }
}

string PolicyRule::getDirectionAsString() const    { return getStr("direction"); }
void   PolicyRule::setDirection(const string& dir) { setStr("direction",dir);    }

bool   PolicyRule::getLogging() const    { return getBool("log"); }
void   PolicyRule::setLogging(bool flag) { setBool("log",flag);   }


void PolicyRule::fromXML(xmlNodePtr root) throw(FWException)
{
    const char* n;

    FWObject::fromXML(root);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("position")));
    if(n)  setInt("position",atoi(n));

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("disabled")));
    if(n)  setStr("disabled",n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("action")));
    if(n)  setStr("action",n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("log")));
    if(n)  setStr("log",n);


    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("interface")));
    if(n)  setStr("interface",n);

    n= FROMXMLCAST(xmlGetProp(root,TOXMLCAST("direction")));
    if(n)  setStr("direction",n);

}

xmlNodePtr PolicyRule::toXML(xmlNodePtr parent) throw(FWException)
{
    xmlNodePtr me = FWObject::toXML(parent, false);
    FWObject *o;
    
    if ( (o=getFirstByType( RuleElementSrc::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementDst::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementSrv::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementInterval::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( PolicyRuleOptions::TYPENAME ))!=NULL )
	o->toXML(me);

    return me;
}

FWOptions* PolicyRule::getOptionsObject()
{
    return FWOptions::cast( getFirstByType(PolicyRuleOptions::TYPENAME) );
}


/***************************************************************************/

const char *NATRule::TYPENAME={"NATRule"};

NATRule::NATRule() : Rule()
{
}

NATRule::NATRule(const FWObject *root) : Rule(root)
{
    FWObject         *re;
    FWObjectDatabase *db=(FWObjectDatabase*)(root);
    assert(db);

    re=db->create("OSrc",true);  assert(re!=NULL); add(re);
    re=db->create("ODst",true);  assert(re!=NULL); add(re);
    re=db->create("OSrv",true);  assert(re!=NULL); add(re);

    re=db->create("TSrc",true);  assert(re!=NULL); add(re);
    re=db->create("TDst",true);  assert(re!=NULL); add(re);
    re=db->create("TSrv",true);  assert(re!=NULL); add(re);

    add( db->create(NATRuleOptions::TYPENAME,true) );
}


RuleElementOSrc*  NATRule::getOSrc()  const
{
    return RuleElementOSrc::cast(getFirstByType(RuleElementOSrc::TYPENAME));
}

RuleElementODst*  NATRule::getODst()  const
{
    return RuleElementODst::cast(getFirstByType(RuleElementODst::TYPENAME));
}

RuleElementOSrv*  NATRule::getOSrv()  const
{
    return RuleElementOSrv::cast(getFirstByType(RuleElementOSrv::TYPENAME));
}

RuleElementTSrc*  NATRule::getTSrc()  const
{
    return RuleElementTSrc::cast(getFirstByType(RuleElementTSrc::TYPENAME));
}

RuleElementTDst*  NATRule::getTDst()  const
{
    return RuleElementTDst::cast(getFirstByType(RuleElementTDst::TYPENAME));
}

RuleElementTSrv*  NATRule::getTSrv()  const
{
    return RuleElementTSrv::cast(getFirstByType(RuleElementTSrv::TYPENAME));
}

RuleElementInterval* NATRule::getWhen()  const
{
    return RuleElementInterval::cast(getFirstByType(RuleElementInterval::TYPENAME));
}



bool NATRule::isEmpty()
{
    RuleElement *osrc=getOSrc();
    RuleElement *odst=getODst();
    RuleElement *osrv=getOSrv();

    RuleElement *tsrc=getTSrc();
    RuleElement *tdst=getTDst();
    RuleElement *tsrv=getTSrv();

    return (osrc->isAny() && odst->isAny() && osrv->isAny() && tsrc->isAny() && tdst->isAny() && tsrv->isAny());
}

void NATRule::fromXML(xmlNodePtr root) throw(FWException)
{
    const char* n;

    FWObject::fromXML(root);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("disabled")));
    if(n)  setStr("disabled",n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("position")));
    if(n)  setStr("position",n);
}

xmlNodePtr NATRule::toXML(xmlNodePtr parent) throw(FWException)
{
    xmlNodePtr me = FWObject::toXML(parent, false);
    FWObject *o;

    if ( (o=getFirstByType( RuleElementOSrc::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementODst::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementOSrv::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementTSrc::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementTDst::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementTSrv::TYPENAME ))!=NULL )
	o->toXML(me);

    if ( (o=getFirstByType( RuleElementInterval::TYPENAME ))!=NULL )
	o->toXML(me);


    if ( (o=getFirstByType( NATRuleOptions::TYPENAME ))!=NULL )
	o->toXML(me);

    return me;
}

FWOptions* NATRule::getOptionsObject()
{
    return FWOptions::cast( getFirstByType(NATRuleOptions::TYPENAME) );
}

NATRule::NATRuleTypes NATRule::getRuleType() const
{ 
    string rt=getRuleTypeAsString();
    if (rt=="SNAT")     return SNAT; 
    if (rt=="DNAT")     return DNAT; 
    if (rt=="Masq")     return Masq; 
    if (rt=="SNetnat")  return SNetnat;
    if (rt=="DNetnat")  return DNetnat;
    if (rt=="Redirect") return Redirect;
    if (rt=="Return")   return Return;
    if (rt=="Skip")     return Skip;
    if (rt=="Continue") return Continue;
    if (rt=="NONAT")    return NONAT;
    return Unknown;
}

void         NATRule::setRuleType(NATRuleTypes rt) 
{ 
    switch (rt) {
    case SNAT:     setRuleType("SNAT");     return;
    case DNAT:     setRuleType("DNAT");     return;
    case Masq:     setRuleType("Masq");     return;
    case SNetnat:  setRuleType("SNetnat");  return;
    case DNetnat:  setRuleType("DNetnat");  return;
    case Redirect: setRuleType("Redirect"); return;
    case Return:   setRuleType("Return");   return;
    case Skip:     setRuleType("Skip");     return;
    case Continue: setRuleType("Continue"); return;
    case NONAT:    setRuleType("NONAT");    return;
    default:       setRuleType("Unknown");  return;
    }
}

string  NATRule::getRuleTypeAsString() const { return getStr("rule_type"); }
void    NATRule::setRuleType(const std::string &rt) { setStr("rule_type",rt); }
